@@ -1,8 +1,6 @@
-/* DASDCOPY.C   (c) Copyright Roger Bowler, 1999-2007                */
+/* DASDCOPY.C   (c) Copyright Roger Bowler, 1999-2010                */
 /*              Copy a dasd file to another dasd file                */
 
-// $Id: dasdcopy.c,v 1.32 2008/11/04 04:50:45 fish Exp $
-
 /*-------------------------------------------------------------------*/
 /*      This program copies a dasd file to another dasd file.        */
 /*      Input file and output file may be compressed or not.         */
@@ -23,26 +21,6 @@
 /*              cfba2fba [-options] ifile [sf=sfile] ofile           */
 /*-------------------------------------------------------------------*/
 
-// $Log: dasdcopy.c,v $
-// Revision 1.32  2008/11/04 04:50:45  fish
-// Ensure consistent utility startup
-//
-// Revision 1.31  2008/11/03 00:03:52  rbowler
-// Program name may be corrupted in dasdcopy messages
-//
-// Revision 1.30  2008/11/02 22:57:01  rbowler
-// Ensure message HHCDC010I is correctly aligned
-//
-// Revision 1.29  2008/04/03 16:16:30  rbowler
-// Standardize comment block in module header (cosmetic change only)
-//
-// Revision 1.28  2007/06/23 00:04:08  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.27  2006/12/08 09:43:19  jj
-// Add CVS message log
-//
-
 #include "hstdinc.h"
 
 #include "hercules.h"
@@ -213,7 +191,7 @@ char            pgmpath[MAX_PATH];      /* prog path in host format  */
     {
         BYTE buf[8];
         hostpath(pathname, ifile, sizeof(pathname));
-        fd = open (pathname, O_RDONLY|O_BINARY);
+        fd = hopen(pathname, O_RDONLY|O_BINARY);
         if (fd < 0)
         {
             fprintf (stderr, _("HHCDC001E %s: %s open error: %s\n"),
@@ -441,7 +419,8 @@ char            pgmpath[MAX_PATH];      /* prog path in host format  */
     }
 
     close_image_file(icif); close_image_file(ocif);
-    printf (_("\r" "HHCDC010I %s successfully completed.\n"), pgm);
+    if (!quiet) printf (_("\r"));
+    printf (_("HHCDC010I %s successfully completed.\n"), pgm);
     return 0;
 }
 
@@ -681,7 +660,7 @@ int syntax (char *pgm)
 #endif
             ,(sizeof(off_t) > 4) ? _( "     -lfs              output ckd file will be a single file\n" ) : ( "" )
             );
-    printf (usage);
+    printf ("%s", usage);
     return -1;
 } /* end function syntax */
 
