@@ -1,16 +1,14 @@
-/* GENERAL2.C   (c) Copyright Roger Bowler, 1994-2007                */
+/* GENERAL2.C   (c) Copyright Roger Bowler, 1994-2009                */
 /*              ESA/390 CPU Emulator                                 */
 /*              Instructions N-Z                                     */
 
-// $Id: general2.c,v 1.126 2008/03/28 13:36:25 rbowler Exp $
+/*              (c) Copyright Peter Kuschnerus, 1999-2009 (UPT & CFC)*/
 
-/*              (c) Copyright Peter Kuschnerus, 1999-2007 (UPT & CFC)*/
-
-/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2007      */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2007      */
+/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2009      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2009      */
 
 /*-------------------------------------------------------------------*/
-/* This module implements all general instructions of the            */
+/* This module implements general instructions N-Z of the            */
 /* S/370 and ESA/390 architectures, as described in the manuals      */
 /* GA22-7000-03 System/370 Principles of Operation                   */
 /* SA22-7201-06 ESA/390 Principles of Operation                      */
@@ -31,74 +29,6 @@
 /*      Clear TEA on data exception - Peter Kuschnerus           v209*/
 /*-------------------------------------------------------------------*/
 
-// $Log: general2.c,v $
-// Revision 1.126  2008/03/28 13:36:25  rbowler
-// Fix incorrect registers when cc=1 for TRTE,TRTRE
-//
-// Revision 1.125  2008/03/27 16:45:17  rbowler
-// TRTE and TRTRE incorrectly return cc=3
-//
-// Revision 1.124  2008/03/06 16:10:35  rbowler
-// Remove extraneous trailing blanks (cosmetic change only)
-//
-// Revision 1.123  2008/03/01 15:15:41  bernard
-// Removed the silly control-M characters
-//
-// Revision 1.122  2008/03/01 12:19:04  rbowler
-// Rename new features to include the word facility
-//
-// Revision 1.121  2008/03/01 07:51:00  bernard
-// TRTE, TRTRE: Checking for a_bit and ch>255 is double. Removed the a_bit.
-//
-// Revision 1.120  2008/02/29 16:25:13  bernard
-// Added Parsing Enhancement Facility
-//
-// Revision 1.119  2008/02/15 21:21:18  ptl00
-// Fix STCKE so that byte 0 is cleared
-//
-// Revision 1.118  2008/01/24 00:59:03  gsmith
-// Fix and optimize TR instruction
-//
-// Revision 1.117  2007/12/30 17:48:21  bernard
-// Anoter UTF xlate error
-//
-// Revision 1.116  2007/12/30 09:09:51  bernard
-// Some errors in UTF translation
-//
-// Revision 1.115  2007/11/30 15:14:14  rbowler
-// Permit String-Instruction facility to be activated in S/370 mode
-//
-// Revision 1.114  2007/06/23 00:04:10  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.113  2007/01/13 07:21:48  bernard
-// backout ccmask
-//
-// Revision 1.112  2007/01/12 15:23:51  bernard
-// ccmask phase 1
-//
-// Revision 1.111  2007/01/09 05:10:19  gsmith
-// Tweaks to lm/stm
-//
-// Revision 1.110  2007/01/04 23:12:04  gsmith
-// remove thunk calls for program_interrupt
-//
-// Revision 1.109  2006/12/31 21:16:32  gsmith
-// 2006 Dec 31 really back out mainlockx.pat
-//
-// Revision 1.108  2006/12/20 09:09:40  jj
-// Fix bogus log entries
-//
-// Revision 1.107  2006/12/20 04:26:20  gsmith
-// 19 Dec 2006 ip_all.pat - performance patch - Greg Smith
-//
-// Revision 1.106  2006/12/20 04:22:00  gsmith
-// 2006 Dec 19 Backout mainlockx.pat - possible SMP problems - Greg Smith
-//
-// Revision 1.105  2006/12/08 09:43:21  jj
-// Add CVS message log
-//
-
 #include "hstdinc.h"
 
 #if !defined(_HENGINE_DLL_)
@@ -212,7 +142,7 @@ int     cc = 0;                         /* Condition code            */
      */
 
     /* Translate addresses of leftmost operand bytes */
-    dest1 = MADDR (addr1, b1, regs, ACCTYPE_WRITE_SKP, regs->psw.pkey);
+    dest1 = MADDRL (addr1, len+1, b1, regs, ACCTYPE_WRITE_SKP, regs->psw.pkey);
     sk1 = regs->dat.storkey;
     source1 = MADDR (addr2, b2, regs, ACCTYPE_READ, regs->psw.pkey);
 
@@ -222,7 +152,7 @@ int     cc = 0;                         /* Condition code            */
         {
             /* (1) - No boundaries are crossed */
             for (i = 0; i <= len; i++)
-                if (*dest1++ |= *source1++) cc = 1;
+                if ((*dest1++ |= *source1++)) cc = 1;
 
         }
         else
@@ -232,10 +162,10 @@ int     cc = 0;                         /* Condition code            */
              source2 = MADDR ((addr2 + len2) & ADDRESS_MAXWRAP(regs),
                               b2, regs, ACCTYPE_READ, regs->psw.pkey);
              for ( i = 0; i < len2; i++)
-                 if (*dest1++ |= *source1++) cc = 1;
+                 if ((*dest1++ |= *source1++)) cc = 1;
              len2 = len - len2;
              for ( i = 0; i <= len2; i++)
-                 if (*dest1++ |= *source2++) cc = 1;
+                 if ((*dest1++ |= *source2++)) cc = 1;
         }
         *sk1 |= (STORKEY_REF | STORKEY_CHANGE);
     }
@@ -251,10 +181,10 @@ int     cc = 0;                         /* Condition code            */
         {
              /* (3) - First operand crosses a boundary */
              for ( i = 0; i < len2; i++)
-                 if (*dest1++ |= *source1++) cc = 1;
+                 if ((*dest1++ |= *source1++)) cc = 1;
              len2 = len - len2;
              for ( i = 0; i <= len2; i++)
-                 if (*dest2++ |= *source1++) cc = 1;
+                 if ((*dest2++ |= *source1++)) cc = 1;
         }
         else
         {
@@ -266,34 +196,34 @@ int     cc = 0;                         /* Condition code            */
             {
                 /* (4a) - Both operands cross at the same time */
                 for ( i = 0; i < len2; i++)
-                    if (*dest1++ |= *source1++) cc = 1;
+                    if ((*dest1++ |= *source1++)) cc = 1;
                 len2 = len - len2;
                 for ( i = 0; i <= len2; i++)
-                    if (*dest2++ |= *source2++) cc = 1;
+                    if ((*dest2++ |= *source2++)) cc = 1;
             }
             else if (len2 < len3)
             {
                 /* (4b) - First operand crosses first */
                 for ( i = 0; i < len2; i++)
-                    if (*dest1++ |= *source1++) cc = 1;
+                    if ((*dest1++ |= *source1++)) cc = 1;
                 len2 = len3 - len2;
                 for ( i = 0; i < len2; i++)
-                    if (*dest2++ |= *source1++) cc = 1;
+                    if ((*dest2++ |= *source1++)) cc = 1;
                 len2 = len - len3;
                 for ( i = 0; i <= len2; i++)
-                    if (*dest2++ |= *source2++) cc = 1;
+                    if ((*dest2++ |= *source2++)) cc = 1;
             }
             else
             {
                 /* (4c) - Second operand crosses first */
                 for ( i = 0; i < len3; i++)
-                    if (*dest1++ |= *source1++) cc = 1;
+                    if ((*dest1++ |= *source1++)) cc = 1;
                 len3 = len2 - len3;
                 for ( i = 0; i < len3; i++)
-                    if (*dest1++ |= *source2++) cc = 1;
+                    if ((*dest1++ |= *source2++)) cc = 1;
                 len3 = len - len2;
                 for ( i = 0; i <= len3; i++)
-                    if (*dest2++ |= *source2++) cc = 1;
+                    if ((*dest2++ |= *source2++)) cc = 1;
             }
         }
         *sk1 |= (STORKEY_REF | STORKEY_CHANGE);
@@ -304,7 +234,7 @@ int     cc = 0;                         /* Condition code            */
 
     ITIMER_UPDATE(addr1,len,regs);
 
-}
+} /* end DEF_INST(or_character) */
 
 
 /*-------------------------------------------------------------------*/
@@ -425,6 +355,7 @@ VADR    effective_addr2,
             break;
 
         default:
+            PTT(PTT_CL_ERR,"*PLO",regs->GR_L(0),regs->GR_L(r1),regs->psw.IA_L);
             /* indicate function not supported */
             regs->psw.cc = 3;
             break;
@@ -549,7 +480,10 @@ VADR    effective_addr2,
         RELEASE_MAINLOCK(regs);
 
         if(regs->psw.cc && sysblk.cpus > 1)
+        {
+            PTT(PTT_CL_CSF,"*PLO",regs->GR_L(0),regs->GR_L(r1),regs->psw.IA_L);
             sched_yield();
+        }
 
     }
 }
@@ -971,7 +905,7 @@ U32    *p1, *p2 = NULL;                 /* Mainstor pointers         */
     m = (0x800 - (effective_addr2 & 0x7ff)) >> 2;
 
     /* Address of operand beginning */
-    p1 = (U32*)MADDR(effective_addr2, b2, regs, ACCTYPE_WRITE, regs->psw.pkey);
+    p1 = (U32*)MADDRL(effective_addr2, n, b2, regs, ACCTYPE_WRITE, regs->psw.pkey);
 
     /* Get address of next page if boundary crossed */
     if (unlikely (m < n))
@@ -1186,6 +1120,7 @@ int     b2;                             /* effective address base    */
 VADR    effective_addr2;                /* effective address         */
 int     i, m, n;                        /* Integer work areas        */
 U32    *p1, *p2;                        /* Mainstor pointers         */
+BYTE   *bp1;                            /* Unaligned mainstor ptr    */
 
     RS(inst, regs, r1, r3, b2, effective_addr2);
 
@@ -1196,14 +1131,28 @@ U32    *p1, *p2;                        /* Mainstor pointers         */
     m = 0x800 - ((VADR_L)effective_addr2 & 0x7ff);
 
     /* Get address of first page */
-    p1 = (U32*)MADDR(effective_addr2, b2, regs, ACCTYPE_WRITE, regs->psw.pkey);
+    bp1 = (BYTE*)MADDRL(effective_addr2, n, b2, regs, ACCTYPE_WRITE, regs->psw.pkey);
+    p1 = (U32*)bp1;
 
     if (likely(n <= m))
     {
         /* boundary not crossed */
         n >>= 2;
-        for (i = 0; i < n; i++)
-            store_fw (p1++, regs->GR_L((r1 + i) & 0xF));
+#if defined(OPTION_STRICT_ALIGNMENT)
+        if(likely(!(((uintptr_t)effective_addr2)&0x03)))
+        {
+#endif
+            for (i = 0; i < n; i++)
+                store_fw (p1++, regs->GR_L((r1 + i) & 0xF));
+#if defined(OPTION_STRICT_ALIGNMENT)
+        }
+        else
+        {
+            for (i = 0; i < n; i++,bp1+=4)
+                store_fw (bp1, regs->GR_L((r1 + i) & 0xF));
+        }
+#endif
+
         ITIMER_UPDATE(effective_addr2,(n*4)-1,regs);
     }
     else
@@ -1406,14 +1355,6 @@ int     rc;                             /* Return code               */
     /* Use the I-byte to set the SVC interruption code */
     regs->psw.intcode = i;
 
-/* OS/390 2.10 debugging for negative length GETMAIN */
-#if 0
-    if ( (i == 0x78) || (i == 0x0a) )
-    {
-        ARCH_DEP(display_inst) (regs, regs->ip);
-    }
-#endif
-
     /* Point to PSA in main storage */
     psa = (void*)(regs->mainstor + px);
 
@@ -1602,7 +1543,7 @@ BYTE   *dest, *dest2 = NULL, *tab, *tab2; /* Mainstor pointers       */
     SS_L(inst, regs, len, b1, addr1, b2, addr2);
 
     /* Get destination pointer */
-    dest = MADDR (addr1, b1, regs, ACCTYPE_WRITE, regs->psw.pkey);
+    dest = MADDRL (addr1, len+1, b1, regs, ACCTYPE_WRITE, regs->psw.pkey);
 
     /* Get pointer to next page if destination crosses a boundary */
     if (CROSS2K (addr1, len))
@@ -1874,10 +1815,9 @@ BYTE    lbyte;                          /* Left result byte of pair  */
 
 /*-------------------------------------------------------------------*/
 /* 0102 UPT   - Update Tree                                      [E] */
-/*              (c) Copyright Peter Kuschnerus, 1999-2007            */
-/*              (c) Copyright "Fish" (David B. Trout), 2005-2007     */
+/*              (c) Copyright Peter Kuschnerus, 1999-2009            */
+/*              (c) Copyright "Fish" (David B. Trout), 2005-2009     */
 /*-------------------------------------------------------------------*/
-
 DEF_INST(update_tree)
 {
 GREG    index;                          /* tree index                */
@@ -1890,8 +1830,6 @@ BYTE    a64 = regs->psw.amode64;        /* 64-bit mode flag          */
 
     E(inst, regs);
 
-    UNREFERENCED(inst);
-
     /*
     **  GR0, GR1    node values (codeword and other data) of node
     **              with "highest encountered codeword value"
@@ -2019,7 +1957,9 @@ BYTE    a64 = regs->psw.amode64;        /* 64-bit mode flag          */
 
     /* Commit GR5 with the actual index value we stopped on */
     SET_GR_A(5,regs,index);
-}
+
+} /* end DEF_INST(update_tree) */
+
 
 #if defined(FEATURE_EXTENDED_TRANSLATION_FACILITY_3)
 /*-------------------------------------------------------------------*/
