@@ -1,60 +1,17 @@
+/* HAO.C        (c) Copyright Bernard van der Helm, 2002-2009        */
+/*             Hercules Automatic Operator Implementation            */
+
 /*---------------------------------------------------------------------------*/
 /* file: hao.c                                                               */
 /*                                                                           */
-/* Implementation of the automatic operator withing the Hercules z/Series    */
-/* emulator. This implementation couldn't be done without the advise of      */
-/* Jan Jaeger, who always has an answer to any question. Thanks for the      */
-/* regex advice!                                                             */
+/* Implementation of the automatic operator within the Hercules emulator.    */
 /*                                                                           */
 /*                            (c) Copyright Bernard van der Helm, 2002-2009  */
 /*                            Noordwijkerhout, The Netherlands.              */
 /*                                                                           */
 /*---------------------------------------------------------------------------*/
 
-// $Id: hao.c 5125 2009-01-23 12:01:44Z bernard $
-//
-// $Log$
-// Revision 1.15  2008/11/30 09:12:41  bernard
-// End the licensed OS experiment
-//
-// Revision 1.14  2008/11/29 12:26:19  bernard
-// PGMPRDOS implementation within hoa. Not perfect! please provide command
-// hoa clear if you want to violate the PGMPRDOS warning.
-//
-// Revision 1.13  2008/11/04 05:56:31  fish
-// Put ensure consistent create_thread ATTR usage change back in
-//
-// Revision 1.12  2008/11/03 15:31:57  rbowler
-// Back out consistent create_thread ATTR modification
-//
-// Revision 1.11  2008/10/18 09:32:21  fish
-// Ensure consistent create_thread ATTR usage
-//
-// Revision 1.10  2008/08/23 11:55:23  fish
-// Increase max #of rules from 10 to 64
-//
-// Revision 1.9  2008/07/30 15:29:04  bernard
-// Strip herc prefix before checking the message.
-//
-// Revision 1.8  2008/07/30 06:25:27  fish
-// use 'if' statement instead of 'min' macro to fix Linux build error,
-// and remove redundant buffer initialization already done by hao_initialize.
-//
-// Revision 1.7  2008/07/29 05:55:41  fish
-// Fix buffer mgmt bug in hao_thread causing garbage
-//
-// Revision 1.6  2008/07/26 14:39:42  bernard
-// Foutje, bedankt!
-//
-// Revision 1.5  2008/07/26 14:30:17  bernard
-// Reject hao automatic commands. This causes deadlocks.
-//
-// Revision 1.4  2007/06/23 00:04:10  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.3  2006/12/08 09:43:21  jj
-// Add CVS message log
-//
+
 
 #include "hstdinc.h"
 
@@ -103,6 +60,7 @@
 
 #define HAO_WKLEN    256    /* (maximum message length able to tolerate) */
 #define HAO_MAXRULE  64     /* (purely arbitrary and easily increasable) */
+#define HAO_MAXCAPT  9      /* (maximum number of capturing groups)      */
 
 /*---------------------------------------------------------------------------*/
 /* local variables                                                           */
@@ -116,7 +74,7 @@ static char     ao_msgbuf[LOG_DEFSIZE+1];   /* (plus+1 for NULL termination) */
 /*---------------------------------------------------------------------------*/
 /* function prototypes                                                       */
 /*---------------------------------------------------------------------------*/
-DLL_EXPORT void  hao_initialize(void);
+DLL_EXPORT int   hao_initialize(void);
 DLL_EXPORT void  hao_command(char *cmd);
 DLL_EXPORT void  hao_message(char *buf);
 static     void  hao_clear(void);
@@ -133,7 +91,7 @@ static     void* hao_thread(void* dummy);
 /* This function is called at system startup by impl.c 'process_rc_file'     */
 /* It initializes all global variables.                                      */
 /*---------------------------------------------------------------------------*/
-DLL_EXPORT void hao_initialize(void)
+DLL_EXPORT int hao_initialize(void)
 {
   int i = 0;
 
@@ -156,11 +114,14 @@ DLL_EXPORT void hao_initialize(void)
   if ( create_thread (&sysblk.haotid, JOINABLE,
     hao_thread, NULL, "hao_thread") )
   {
-    logmsg(_("HHCIN004S Cannot create HAO thread: %s\n"),
-      strerror(errno));
+    i = FALSE;
   }
+  else
+    i = TRUE;
 
   release_lock(&ao_lock);
+
+  return(i);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -297,7 +258,7 @@ static void hao_tgt(char *arg)
   }
 
   /* compile the target string */
-  rc = regcomp(&ao_preg[i], arg, 0);
+  rc = regcomp(&ao_preg[i], arg, REG_EXTENDED);
 
   /* check for error */
   if(rc)
@@ -614,6 +575,31 @@ static void* hao_thread(void* dummy)
   return NULL;
 }
 
+/*---------------------------------------------------------------------------*/
+/* size_t hao_subst(char *str, size_t soff, size_t eoff,                     */
+/*              char *cmd, size_t coff, size_t csize)                        */
+/*                                                                           */
+/* This function copies a substring of the original string into              */
+/* the command buffer. The input parameters are:                             */
+/*      str     the original string                                          */
+/*      soff    starting offset of the substring in the original string      */
+/*      eoff    offset of the first character following the substring        */
+/*      cmd     the destination command buffer                               */
+/*      coff    offset in the command buffer to copy the substring           */
+/*      csize   size of the command buffer (including terminating zero)      */
+/* The return value is the number of characters copied.                      */
+/*---------------------------------------------------------------------------*/
+static size_t hao_subst(char *str, size_t soff, size_t eoff,
+        char *cmd, size_t coff, size_t csize)
+{
+  size_t len = eoff - soff;
+
+  if (soff + len > strlen(str)) len = strlen(str) - soff;
+  if (coff + len > csize-1) len = csize-1 - coff;
+  memcpy(cmd + coff, str + soff, len);
+  return len;
+}
+
 /*---------------------------------------------------------------------------*/
 /* void hao_message(char *buf)                                               */
 /*                                                                           */
@@ -624,8 +610,11 @@ static void* hao_thread(void* dummy)
 DLL_EXPORT void hao_message(char *buf)
 {
   char work[HAO_WKLEN];
-  regmatch_t rm;
-  int i;
+  char cmd[HAO_WKLEN];
+  regmatch_t rm[HAO_MAXCAPT+1];
+  int i, j, k, numcapt;
+  size_t n;
+  char *p;
 
   /* copy and strip spaces */
   hao_cpstrp(work, buf);
@@ -655,11 +644,55 @@ DLL_EXPORT void hao_message(char *buf)
     if(ao_tgt[i] && ao_cmd[i])  /* complete rule defined in this slot? */
     {
       /* does this rule match our message? */
-      if (regexec(&ao_preg[i], work, 1, &rm, 0) == 0)
+      if (regexec(&ao_preg[i], work, HAO_MAXCAPT+1, rm, 0) == 0)
       {
+        /* count the capturing group matches */
+        for (j = 0; j <= HAO_MAXCAPT && rm[j].rm_so >= 0; j++);
+        numcapt = j - 1;
+
+        /* copy the command and process replacement patterns */
+        for (n=0, p=ao_cmd[i]; *p && n < sizeof(cmd)-1; ) {
+          /* replace $$ by $ */
+          if (*p == '$' && p[1] == '$') {
+            cmd[n++] = '$';
+            p += 2;
+            continue;
+          }
+          /* replace $` by characters to the left of the match */
+          if (*p == '$' && p[1] == '`') {
+            n += hao_subst(work, 0, rm[0].rm_so, cmd, n, sizeof(cmd));
+            p += 2;
+            continue;
+          }
+          /* replace $' by characters to the right of the match */
+          if (*p == '$' && p[1] == '\'') {
+            n += hao_subst(work, rm[0].rm_eo, strlen(work), cmd, n, sizeof(cmd));
+            p += 2;
+            continue;
+          }
+          /* replace $1..$99 by the corresponding capturing group */
+          if (*p == '$' && isdigit(p[1])) {
+            if (isdigit(p[2])) {
+              j = (p[1]-'0') * 10 + (p[2]-'0');
+              k = 3;
+            } else {
+              j = p[1]-'0';
+              k = 2;
+            }
+            if (j > 0 && j <= numcapt) {
+              n += hao_subst(work, rm[j].rm_so, rm[j].rm_eo, cmd, n, sizeof(cmd));
+              p += k;
+              continue;
+            }
+          }
+          /* otherwise copy one character */
+          cmd[n++] = *p++;
+        }
+        cmd[n] = '\0';
+
         /* issue command for this rule */
-        logmsg(HHCAO003I, ao_cmd[i]);
-        panel_command(ao_cmd[i]);
+        logmsg(HHCAO003I, cmd);
+        panel_command(cmd);
       }
     }
   }
@@ -668,41 +701,3 @@ DLL_EXPORT void hao_message(char *buf)
 
 #endif /* defined(OPTION_HAO) */
 
-///////////////////////////////////////////////////////////////////////////////
-// Fish notes for possible future enhancement: I was thinking we should
-// probably at some point support command rules that allow a custom
-// command to be constructed based on the string(s) that was/were found.
-
-#if 0
-    /*
-        The following demonstrates how the REG_NOTBOL flag could be used
-        with regexec() to find all substrings in a line that match a pattern
-        supplied by a user. (For simplicity of the example, very little
-        error checking is done.) 
-    */
-
-    regex_t     re;         /*  regular expression  */
-    regmatch_t  pm;         /*  match info          */
-    int         error;      /*  rc from regexec     */
-
-    char        buffer[]   =  "The quick brown dog jumps over the lazy fox";
-    char        pattern[]  =  "lazy";
-
-    /*  Compile the search pattern  */
-
-    (void)regcomp( &re, pattern, 0 );
-
-    /*  This call to regexec() finds the first match on the line  */
-
-    error = regexec( &re, &buffer[0], 1, &pm, 0 );
-
-    while (error == 0)      /* while matches found */
-    {
-        /*  Substring found between 'pm.rm_so' and 'pm.rm_eo'  */
-        /*  This call to regexec() finds the next match    */
-
-        error = regexec( &re, buffer + pm.rm_eo, 1, &pm, REG_NOTBOL );
-    }
-
-#endif
-///////////////////////////////////////////////////////////////////////////////
