@@ -2,66 +2,6 @@
 /*              (c) Copyright Jan Jaeger, 1999-2009                  */
 /*              Miscellaneous System Command Routines                */
 
-// $Id: hscmisc.c 5125 2009-01-23 12:01:44Z bernard $
-//
-// $Log$
-// Revision 1.68  2008/11/04 05:56:31  fish
-// Put ensure consistent create_thread ATTR usage change back in
-//
-// Revision 1.67  2008/11/03 15:31:55  rbowler
-// Back out consistent create_thread ATTR modification
-//
-// Revision 1.66  2008/10/18 09:32:21  fish
-// Ensure consistent create_thread ATTR usage
-//
-// Revision 1.65  2008/05/11 22:30:37  rbowler
-// V command should display "dat off" instead of "primary" if addr is real
-//
-// Revision 1.64  2008/04/09 13:53:45  rbowler
-// Operand disassembly for RIL instructions
-//
-// Revision 1.63  2008/04/09 09:09:22  bernard
-// EXRL instruction
-//
-// Revision 1.62  2008/03/07 17:46:42  ptl00
-// Add pri, sec, home options to v command
-//
-// Revision 1.61  2008/02/19 11:49:19  ivan
-// - Move setting of CPU priority after spwaning timer thread
-// - Added support for Posix 1003.1e capabilities
-//
-// Revision 1.60  2007/06/23 00:04:11  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.59  2007/03/17 22:20:46  gsmith
-// Fix hostregs address calculation in copy_regs
-//
-// Revision 1.58  2007/03/16 22:47:10  gsmith
-// Reduce REGS copying by hscmisc.c
-//
-// Revision 1.57  2007/01/07 11:25:33  rbowler
-// Instruction tracing regsfirst and noregs modes
-//
-// Revision 1.56  2007/01/06 09:05:18  gsmith
-// Enable display_inst to display traditionally too
-//
-// Revision 1.55  2006/12/30 18:47:30  fish
-// 1. Display regs BEFORE instr being traced.
-// 2. Fix condition for Control Regs trace
-//
-// Revision 1.54  2006/12/18 14:01:54  rbowler
-// Only show CPU in FPR display if numcpu>1
-//
-// Revision 1.53  2006/12/17 23:03:28  rbowler
-// Display FPR when tracing floating-point instructions
-//
-// Revision 1.52  2006/12/17 21:58:50  rbowler
-// FPR command to display register pairs
-//
-// Revision 1.51  2006/12/08 09:43:26  jj
-// Add CVS message log
-//
-
 #include "hstdinc.h"
 
 #if !defined(_HENGINE_DLL_)
@@ -223,9 +163,6 @@ static void do_shutdown_now()
 #endif /*defined(OPTION_DYNAMIC_LOAD)*/
        )
     {
-#if defined(FISH_HANG)
-        FishHangAtExit();
-#endif
 #ifdef _MSVC_
         socket_deinit();
 #endif
@@ -260,14 +197,23 @@ static void do_shutdown_wait()
 void do_shutdown()
 {
 TID tid;
-    if(is_wait_sigq_pending())
-        cancel_wait_sigq();
+#if defined(_MSVC_)
+    if ( sysblk.shutimmed )
+        do_shutdown_now();
     else
-        if(can_signal_quiesce() && !signal_quiesce(0,0))
-            create_thread(&tid, DETACHED, do_shutdown_wait,
-                          NULL, "do_shutdown_wait");
+    {
+#endif // defined(_MSVC_)
+        if(is_wait_sigq_pending())
+            cancel_wait_sigq();
         else
-            do_shutdown_now();
+            if(can_signal_quiesce() && !signal_quiesce(0,0))
+                create_thread(&tid, DETACHED, do_shutdown_wait,
+                              NULL, "do_shutdown_wait");
+            else
+                do_shutdown_now();
+#if defined(_MSVC_)
+    }
+#endif // defined(_MSVC_)
 }
 /*-------------------------------------------------------------------*/
 /* The following 2 routines display an array of 32/64 registers      */
@@ -350,7 +296,7 @@ static void display_regs64(char *hdr,U16 cpuad,U64 *r,int numcpus)
 void display_inst_regs (REGS *regs, BYTE *inst, BYTE opcode)
 {
     /* Display the general purpose registers */
-    if (!(opcode == 0xB3 || (opcode >= 0x20 && opcode <= 0x3F)) 
+    if (!(opcode == 0xB3 || (opcode >= 0x20 && opcode <= 0x3F))
         || (opcode == 0xB3 && (
                 (inst[1] >= 0x80 && inst[1] <= 0xCF)
                 || (inst[1] >= 0xE1 && inst[1] <= 0xFE)
@@ -698,7 +644,7 @@ static REGS  *copy_regs (REGS *regs)
  REGS  *newregs, *hostregs;
  size_t size;
 
-    size = SIE_MODE(regs) ? 2*sizeof(REGS) : sizeof(REGS);
+    size = (SIE_MODE(regs) || SIE_ACTIVE(regs)) ? 2*sizeof(REGS) : sizeof(REGS);
     newregs = malloc(size);
     if (newregs == NULL)
     {
@@ -713,8 +659,10 @@ static REGS  *copy_regs (REGS *regs)
     newregs->ghostregs = 1;
     newregs->hostregs = newregs;
     newregs->guestregs = NULL;
+    newregs->sie_active=0;
 
     /* Copy host regs if in SIE mode */
+    /* newregs is a SIE Guest REGS */
     if(SIE_MODE(newregs))
     {
         hostregs = newregs + 1;
@@ -754,8 +702,8 @@ static REGS  *copy_regs (REGS *regs)
 /*      if a translation exception occurs), the translation is       */
 /*      performed using a temporary copy of the CPU registers.       */
 /*-------------------------------------------------------------------*/
-static U16 ARCH_DEP(virt_to_abs) (RADR *raptr, int *siptr,
-                        VADR vaddr, int arn, REGS *regs, int acctype)
+static U16 ARCH_DEP(virt_to_abs) (RADR *raptr, int *siptr, VADR vaddr,
+                            volatile int arn, REGS *regs, int acctype)
 {
 int icode;
 
@@ -797,7 +745,7 @@ BYTE    c;                              /* Character work area       */
     if(ITIMER_ACCESS(raddr,16))
         ARCH_DEP(store_int_timer)(regs);
 #endif
-    
+
     if (draflag)
     {
         n = sprintf (buf, "R:"F_RADR":", raddr);
@@ -877,6 +825,7 @@ BYTE    inst[6];                        /* Storage alteration value  */
 BYTE    opcode;
 U16     xcode;
 char    type;
+char    buf[80];
 
     /* Set limit for address range */
   #if defined(FEATURE_ESAME)
@@ -954,7 +903,8 @@ char    type;
         }
         else
             logmsg("         ");
-        DISASM_INSTRUCTION(inst);
+        DISASM_INSTRUCTION(inst, buf);
+        logmsg("%s\n", buf);
         saddr += ilc;
     } /* end for(i) */
 
@@ -1189,8 +1139,8 @@ REGS   *regs;                           /* Copied regs               */
     if (ilc > 2) n += sprintf (buf+n, "%2.2X%2.2X", inst[2], inst[3]);
     if (ilc > 4) n += sprintf (buf+n, "%2.2X%2.2X", inst[4], inst[5]);
     logmsg ("%s %s", buf,(ilc<4) ? "        " : (ilc<6) ? "    " : "");
-    DISASM_INSTRUCTION(inst);
-
+    DISASM_INSTRUCTION(inst, buf);
+    logmsg("%s\n", buf);
 
 #ifdef DISPLAY_INSTRUCTION_OPERANDS
 
@@ -1278,7 +1228,7 @@ REGS   *regs;                           /* Copied regs               */
         || opcode == 0xC6)
     {
         S64 offset = 2LL*(S32)(fetch_fw(inst+2));
-        addr1 = (likely(!regs->execflag)) ? 
+        addr1 = (likely(!regs->execflag)) ?
                         PSW_IA(regs, offset) : \
                         (regs->ET + offset) & ADDRESS_MAXWRAP(regs);
         b1 = 0;
@@ -1292,7 +1242,7 @@ REGS   *regs;                           /* Copied regs               */
                                                 ACCTYPE_READ);
         else
             n = ARCH_DEP(display_virt) (regs, addr1, buf, b1,
-                                (opcode == 0x44 
+                                (opcode == 0x44
 #if defined(FEATURE_EXECUTE_EXTENSIONS_FACILITY)
                                  || (opcode == 0xc6 && !(inst[1] & 0x0f))
 #endif /*defined(FEATURE_EXECUTE_EXTENSIONS_FACILITY)*/
@@ -1484,7 +1434,7 @@ int herc_system (char* command)
 
   #define  SHELL_CMD_SHIM_PGM   "conspawn "
 
-    int rc = strlen(SHELL_CMD_SHIM_PGM) + strlen(command) + 1;
+    int rc = (int)(strlen(SHELL_CMD_SHIM_PGM) + strlen(command) + 1);
     char* pszNewCommandLine = malloc( rc );
     strlcpy( pszNewCommandLine, SHELL_CMD_SHIM_PGM, rc );
     strlcat( pszNewCommandLine, command,            rc );
